#undef OPENTRACKER_EXPORTS
#include <OpenTracker/OpenTracker.h>
#include <OpenTracker/input/SPLModules.h>
#include <OpenTracker/input/MyTutorialModule.h>
#include <iostream>
#include <fstream>
#include <exception>
#include <stdlib.h>
#include <string.h>

#include <OpenTracker/types/Image.h>

#include "pgmimage.h"
#ifdef _HAVE_SIMPLE_CANVAS
 #include "simplecanvas.h"
#endif //_HAVE_SIMPLE_CANVAS

#define MAX_NUM_IMAGE 20

using namespace std;
using namespace ot;

#define DEFAULT_LOOP_RATE    100

int main(int argc, char **argv)
{
  char* xmlfile;
  long  rate;

  // set parameters based on arguments
  if (argc <= 1) {
    cerr << "Usage: " << argv[0] << " <OT XML> [<rate in ms>]"  << endl;
    exit(-1);
  }
  xmlfile = argv[1];
  if (argc > 2) {
    rate = atoi(argv[2]) * 1000;
  } else {
    rate = DEFAULT_LOOP_RATE * 1000;
  }
  cerr << "XML File : " << xmlfile << endl;
  cerr << "Loop rate: " << (rate/1000) << "ms" << endl;


  // context loop
  addSPLModules();

  Context context(1);
  context.parseConfiguration(xmlfile);
  context.start();

  MyTutorialModule* mtm = (MyTutorialModule*) context.getModule("MyTutorialConfig");

  int stopflag = 0;
  int i = 0;

#ifdef _HAVE_SIMPLE_CANVAS
  SimpleCanvas sc(256, 256);
  sc.run();
#endif //_HAVE_SIMPLE_CANVAS

  while (stopflag == 0) {

    stopflag = context.loopOnce();

    if (mtm) {
      Image img;
      int w, h;

      mtm->getImage(img, w, h);
      if (img.size() != 0) {

        PgmImage pi(w, h, (short*)img.image_ptr);

        if (w > 0 && h > 0) {
#ifdef _HAVE_SIMPLE_CANVAS        
          sc.load((unsigned char*)pi.getDataC());
          sc.update();
#else
          char filename[50];
          sprintf(filename, "out-%03d.pgm", i);
          std::ofstream fout(filename);
          fout << pi;
          fout.close();
#endif //_HAVE_SIMPLE_CANVAS
          i ++;
          i = i % MAX_NUM_IMAGE;
        }
      }
    }

    usleep(rate);
  }

#ifdef _HAVE_SIMPLE_CANVAS        
  sc.stop();
#endif
  context.close();

}
