#undef OPENTRACKER_EXPORTS
#include <OpenTracker/OpenTracker.h>
#include <OpenTracker/input/SPLModules.h>
#include <OpenTracker/input/MyTutorialModule.h>
#include <iostream>
#include <exception>
#include <stdlib.h>

using namespace std;
using namespace ot;


#define DEFAULT_LOOP_RATE    100

int main(int argc, char **argv)
{
  char* xmlfile;
  long  rate;

  // set parameters based on arguments
  if (argc <= 1) {
    cerr << "Usage: " << argv[0] << " <OT XML> [<rate in ms>]"  << endl;
    exit(-1);
  }
  xmlfile = argv[1];
  if (argc > 2) {
    rate = atoi(argv[2]) * 1000;
  } else {
    rate = DEFAULT_LOOP_RATE * 1000;
  }
  cerr << "XML File : " << xmlfile << endl;
  cerr << "Loop rate: " << (rate/1000) << "ms" << endl;


  // context loop

  addSPLModules();

  Context context(1);
  context.parseConfiguration(xmlfile);
  context.start();

  MyTutorialModule* mtm = (MyTutorialModule*) context.getModule("MyTutorialConfig");

  int stopflag = 0;

  std::vector<float> pos(3, 0.0);
  std::vector<float> quat(4, 0.0);

  while (stopflag == 0) {

    stopflag = context.loopOnce();

    if (mtm) {
      mtm->getTracker(pos, quat);
      std::cout << "Received position = ("
                << pos[0] << ", " << pos[1] << ", " << pos[2] << "),  ";
      std::cout << "orientation = ("
                << quat[0] << ", " << quat[1] << ", " << quat[2] << ","
                << quat[3] << ")"  << std::endl;
    }

    usleep(rate);
  }

  context.close();

}



